@@ -20,12 +20,16 @@
 #include <libgnome/libgnome.h>
 #include <syslog.h>
 #include <unistd.h>
+#include <dirent.h>
 #include <signal.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
+#include <sys/time.h>
+#include <sys/resource.h>
 #include <fcntl.h>
 #include <netinet/in.h>
 #include <net/if.h>
@@ -33,6 +37,7 @@
 #include <netdb.h>
 #include <errno.h>
 #include <grp.h>
+#include <pwd.h>
 #include <sys/types.h>
 #ifdef HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>
@@ -42,6 +47,7 @@
 
 #include "gdm.h"
 #include "misc.h"
+#include "xdmcp.h"
 #include "slave.h"
 
 /* Configuration option variables */
@@ -59,6 +65,8 @@ extern pid_t gdm_main_pid;
 extern gboolean preserve_ld_vars;
 extern int gdm_in_signal;
 
+extern gboolean no_console;
+
 extern char *gdm_charset;
 
 static void
@@ -190,7 +198,7 @@ gdm_debug (const gchar *format, ...)
     va_list args;
     char *s;
 
-    if ( ! GdmDebug) 
+    if G_LIKELY ( ! GdmDebug) 
 	return;
 
     va_start (args, format);
@@ -208,12 +216,28 @@ gdm_fdprintf (int fd, const gchar *format, ...)
 {
 	va_list args;
 	gchar *s;
+	int written, len;
 
 	va_start (args, format);
 	s = g_strdup_vprintf (format, args);
 	va_end (args);
 
-	write (fd, s, strlen (s));
+	len = strlen (s);
+
+	if (len == 0) {
+		g_free (s);
+		return;
+	}
+
+	written = 0;
+	while (written < len) {
+		int w;
+		IGNORE_EINTR (w = write (fd, &s[written], len - written));
+		if (w < 0)
+			/* evil! */
+			break;
+		written += w;
+	}
 
 	g_free (s);
 }
@@ -320,6 +344,7 @@ gdm_get_free_display (int start, uid_t server_uid)
 		struct stat s;
 		char buf[256];
 		FILE *fp;
+		int r;
 
 		for (li = displays; li != NULL; li = li->next) {
 			GdmDisplay *dsp = li->data;
@@ -337,17 +362,19 @@ gdm_get_free_display (int start, uid_t server_uid)
 		serv_addr.sin_port = htons (6000 + i);
 
 		errno = 0;
-		if (connect (sock, (struct sockaddr *)&serv_addr,
-			     sizeof (serv_addr)) >= 0 ||
-		    errno != ECONNREFUSED) {
-			close (sock);
+		IGNORE_EINTR (connect (sock,
+				       (struct sockaddr *)&serv_addr,
+				       sizeof (serv_addr)));
+		if (errno != 0 && errno != ECONNREFUSED) {
+			IGNORE_EINTR (close (sock));
 			continue;
 		}
-		close (sock);
+		IGNORE_EINTR (close (sock));
 
 		/* if lock file exists and the process exists */
 		g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", i);
-		if (stat (buf, &s) == 0 &&
+		IGNORE_EINTR (r = stat (buf, &s));
+		if (r == 0 &&
 		    ! S_ISREG (s.st_mode)) {
 			/* Eeeek! not a regular file?  Perhaps someone
 			   is trying to play tricks on us */
@@ -368,7 +395,7 @@ gdm_get_free_display (int start, uid_t server_uid)
 			fclose (fp);
 
 			/* whack the file, it's a stale lock file */
-			unlink (buf);
+			IGNORE_EINTR (unlink (buf));
 		}
 
 		/* if starting as root, we'll be able to overwrite any
@@ -377,14 +404,16 @@ gdm_get_free_display (int start, uid_t server_uid)
 		if (server_uid > 0) {
 			g_snprintf (buf, sizeof (buf),
 				    "/tmp/.X11-unix/X%d", i);
-			if (stat (buf, &s) == 0 &&
+			IGNORE_EINTR (r = stat (buf, &s));
+			if (r == 0 &&
 			    s.st_uid != server_uid) {
 				continue;
 			}
 
 			g_snprintf (buf, sizeof (buf),
 				    "/tmp/.X%d-lock", i);
-			if (stat (buf, &s) == 0 &&
+			IGNORE_EINTR (r = stat (buf, &s));
+			if (r == 0 &&
 			    s.st_uid != server_uid) {
 				continue;
 			}
@@ -402,6 +431,9 @@ gdm_text_message_dialog (const char *msg)
 	char *dialog; /* do we have dialog?*/
 	char *msg_quoted;
 
+	if (no_console)
+		return FALSE;
+
 	if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) != 0)
 		return FALSE;
 
@@ -419,7 +451,7 @@ gdm_text_message_dialog (const char *msg)
 		argv[1] = "-l";
 		argv[2] = "/bin/sh";
 		argv[3] = "-c";
-		argv[4] = g_strdup_printf ("%s --msgbox %s 11 70",
+		argv[4] = g_strdup_printf ("%s --msgbox %s 16 70",
 					   dialog, msg_quoted);
 		argv[5] = NULL;
 
@@ -428,10 +460,12 @@ gdm_text_message_dialog (const char *msg)
 				   TRUE /* de_setuid */) < 0) {
 			g_free (dialog);
 			g_free (msg_quoted);
+			g_free (argv[4]);
 			return FALSE;
 		}
 
 		g_free (dialog);
+		g_free (argv[4]);
 	} else {
 		char *argv[6];
 
@@ -447,11 +481,11 @@ gdm_text_message_dialog (const char *msg)
 
 		if (gdm_exec_wait (argv, TRUE /* no display */,
 				   TRUE /* de_setuid */) < 0) {
-			g_free (argv[3]);
+			g_free (argv[4]);
 			g_free (msg_quoted);
 			return FALSE;
 		}
-		g_free (argv[3]);
+		g_free (argv[4]);
 	}
 	g_free (msg_quoted);
 	return TRUE;
@@ -462,6 +496,9 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 {
 	char *dialog; /* do we have dialog?*/
 	char *msg_quoted;
+
+	if (no_console)
+		return FALSE;
 	
 	if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) != 0)
 		return FALSE;
@@ -484,7 +521,7 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 		argv[1] = "-l";
 		argv[2] = "/bin/sh";
 		argv[3] = "-c";
-		argv[4] = g_strdup_printf ("%s --yesno %s 11 70",
+		argv[4] = g_strdup_printf ("%s --yesno %s 16 70",
 					   dialog, msg_quoted);
 		argv[5] = NULL;
 
@@ -493,6 +530,7 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 		retint = gdm_exec_wait (argv, TRUE /* no display */,
 					TRUE /* de_setuid */);
 		if (retint < 0) {
+			g_free (argv[4]);
 			g_free (dialog);
 			g_free (msg_quoted);
 			return FALSE;
@@ -503,6 +541,7 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 
 		g_free (dialog);
 		g_free (msg_quoted);
+		g_free (argv[4]);
 
 		return TRUE;
 	} else {
@@ -518,7 +557,7 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 			return FALSE;
 		}
 
-		close (tempfd);
+		IGNORE_EINTR (close (tempfd));
 
 		argv[0] = EXPANDED_LIBEXECDIR "/gdmopen";
 		argv[1] = "-l";
@@ -536,11 +575,11 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 
 		if (gdm_exec_wait (argv, TRUE /* no display */,
 				   TRUE /* de_setuid */) < 0) {
-			g_free (argv[3]);
+			g_free (argv[4]);
 			g_free (msg_quoted);
 			return FALSE;
 		}
-		g_free (argv[3]);
+		g_free (argv[4]);
 
 		if (ret != NULL) {
 			fp = fopen (tempname, "r");
@@ -555,7 +594,7 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 			}
 		}
 
-		unlink (tempname);
+		IGNORE_EINTR (unlink (tempname));
 
 		g_free (msg_quoted);
 		return TRUE;
@@ -578,7 +617,7 @@ gdm_exec_wait (char * const *argv, gboolean no_display,
 	if (pid == 0) {
 		closelog ();
 
-		gdm_close_all_descriptors (0 /* from */, -1 /* except */);
+		gdm_close_all_descriptors (0 /* from */, -1 /* except */, -1 /* except2 */);
 
 		/* No error checking here - if it's messed the best response
 		 * is to ignore & try to continue */
@@ -597,7 +636,7 @@ gdm_exec_wait (char * const *argv, gboolean no_display,
 			ve_unsetenv ("XAUTHORITY");
 		}
 		
-		execv (argv[0], argv);
+		IGNORE_EINTR (execv (argv[0], argv));
 
 		_exit (-1);
 	}
@@ -741,7 +780,7 @@ gdm_wait_for_extra (int *status)
 {
 	gdm_sigchld_block_push ();
 
-	if (extra_process > 0) {
+	if (extra_process > 1) {
 		ve_waitpid_no_signal (extra_process, &extra_status, 0);
 	}
 	extra_process = 0;
@@ -752,34 +791,49 @@ gdm_wait_for_extra (int *status)
 	gdm_sigchld_block_pop ();
 }
 
-/* done before each login.  This can do so sanity ensuring,
- * one of the things it does now is make sure /tmp/.ICE-unix
- * exists and has the correct permissions */
+/* done before each login and on startup.  This can do some
+ * sanity ensuring, one of the things it does now is make
+ * sure /tmp/.ICE-unix exists and has the correct permissions */
 void
 gdm_ensure_sanity (void)
 {
 	mode_t old_umask;
+	uid_t old_euid;
+	gid_t old_egid;
+
+	old_euid = geteuid ();
+	old_egid = getegid ();
+
+	seteuid (0);
+	setegid (0);
 
 	/* The /tmp/.ICE-unix check, note that we do
 	 * ignore errors, since it's not deadly to run
 	 * if we can't perform this task :) */
 	old_umask = umask (0);
 
-        if (mkdir ("/tmp/.ICE-unix", 0777) == 0) {
-		/* Make sure it is root */
-		if (chown ("/tmp/.ICE-unix", 0, 0) == 0)
-			chmod ("/tmp/.ICE-unix", 01777);
-        } else {
+        if G_UNLIKELY (mkdir ("/tmp/.ICE-unix", 01777) != 0) {
+		/* if we can't create it, perhaps it
+		   already exists, in which case ensure the
+		   correct permissions */
 		struct stat s;
-		if (lstat ("/tmp/.ICE-unix", &s) == 0 &&
-		    S_ISDIR (s.st_mode)) {
+		int r;
+		IGNORE_EINTR (r = lstat ("/tmp/.ICE-unix", &s));
+		if G_LIKELY (r == 0 && S_ISDIR (s.st_mode)) {
 			/* Make sure it is root and sticky */
-			if (chown ("/tmp/.ICE-unix", 0, 0) == 0)
-				chmod ("/tmp/.ICE-unix", 01777);
+			IGNORE_EINTR (chown ("/tmp/.ICE-unix", 0, 0));
+			IGNORE_EINTR (chmod ("/tmp/.ICE-unix", 01777));
+		} else {
+			/* There is a file/link/whatever called .ICE-unix? whack and try mkdir */
+			IGNORE_EINTR (unlink ("/tmp/.ICE-unix"));
+			mkdir ("/tmp/.ICE-unix", 01777);
 		}
 	}
 
 	umask (old_umask);
+
+	seteuid (old_euid);
+	setegid (old_egid);
 }
 
 const GList *
@@ -828,12 +882,12 @@ gdm_peek_local_address_list (void)
 
 	ifc.ifc_len = sizeof(struct ifreq) * num;
 	ifc.ifc_buf = buf = g_malloc0 (ifc.ifc_len);
-	if (ioctl (sockfd, SIOCGIFCONF, &ifc) < 0) {
+	if G_UNLIKELY (ioctl (sockfd, SIOCGIFCONF, &ifc) < 0) {
 		gdm_error (_("%s: Cannot get local addresses!"),
 			   "gdm_peek_local_address_list");
 		g_free (buf);
 		if (sockfd != gdm_xdmcpfd)
-			close (sockfd);
+			IGNORE_EINTR (close (sockfd));
 		addr = g_new0 (struct in_addr, 1);
 		addr->s_addr = INADDR_LOOPBACK;
 		return g_list_append (NULL, addr);
@@ -865,14 +919,14 @@ gdm_peek_local_address_list (void)
 	}
 
 	if (sockfd != gdm_xdmcpfd)
-		close (sockfd);
+		IGNORE_EINTR (close (sockfd));
 
 	g_free (buf);
 #else /* SIOCGIFCONF */
 	/* host based fallback, will likely only get 127.0.0.1 i think */
 
 	hostbuf[BUFSIZ-1] = '\0';
-	if (gethostname (hostbuf, BUFSIZ-1) != 0) {
+	if G_UNLIKELY (gethostname (hostbuf, BUFSIZ-1) != 0) {
 		gdm_debug ("%s: Could not get server hostname: %s!",
 			   "gdm_peek_local_address_list",
 			   strerror (errno));
@@ -881,7 +935,7 @@ gdm_peek_local_address_list (void)
 		return g_list_append (NULL, addr);
 	}
 	he = gethostbyname (hostbuf);
-	if (he == NULL) {
+	if G_UNLIKELY (he == NULL) {
 		gdm_debug ("%s: Could not get address from hostname!",
 			   "gdm_peek_local_address_list");
 		addr = g_new0 (struct in_addr, 1);
@@ -943,12 +997,12 @@ gdm_setup_gids (const char *login, gid_t gid)
 {
 	/* FIXME: perhaps for *BSD there should be setusercontext
 	 * stuff here */
-	if (setgid (gid) < 0)  {
+	if G_UNLIKELY (setgid (gid) < 0)  {
 		gdm_error (_("Could not setgid %d. Aborting."), (int)gid);
 		return FALSE;
 	}
 
-	if (initgroups (login, gid) < 0) {
+	if G_UNLIKELY (initgroups (login, gid) < 0) {
 		gdm_error (_("initgroups() failed for %s. Aborting."), login);
 		return FALSE;
 	}
@@ -1054,7 +1108,7 @@ gdm_fdgetc (int fd)
 	char buf[1];
 	int bytes;
 
-	bytes = read (fd, buf, 1);
+	IGNORE_EINTR (bytes = read (fd, buf, 1));
 	if (bytes != 1)
 		return EOF;
 	else
@@ -1087,13 +1141,51 @@ gdm_fdgets (int fd)
 }
 
 void
-gdm_close_all_descriptors (int from, int except)
+gdm_close_all_descriptors (int from, int except, int except2)
 {
-	int i;
-	int max = sysconf (_SC_OPEN_MAX);
-	for (i = from; i < max; i++) {
-		if (i != except)
-			close(i);
+	DIR *dir;
+	struct dirent *ent;
+	GSList *openfds = NULL;
+
+	/* evil, but less evil then going to _SC_OPEN_MAX
+	   which can be very VERY large */
+	dir = opendir ("/proc/self/fd/"); /* This is the Linux dir */
+	if (dir == NULL)
+		dir = opendir ("/dev/fd/"); /* This is the FreeBSD dir */
+	if G_LIKELY (dir != NULL) {
+		GSList *li;
+		while ((ent = readdir (dir)) != NULL) {
+			int fd;
+			if (ent->d_name[0] == '.')
+				continue;
+			fd = atoi (ent->d_name);
+			if (fd >= from && fd != except && fd != except2)
+				openfds = g_slist_prepend (openfds, GINT_TO_POINTER (fd));
+		}
+		closedir (dir);
+		for (li = openfds; li != NULL; li = li->next) {
+			int fd = GPOINTER_TO_INT (li->data); 
+			IGNORE_EINTR (close(fd));
+		}
+		g_slist_free (openfds);
+	} else {
+		int i;
+		int max = sysconf (_SC_OPEN_MAX);
+		/* don't go higher then this.  This is
+		 * a safety measure to not hang on crazy
+		 * systems */
+		if G_UNLIKELY (max > 4096) {
+			/* FIXME: warn about this perhaps */
+			/* try an open, in case we're really
+			   leaking fds somewhere badly, this
+			   should be very high */
+			i = gdm_open_dev_null (O_RDONLY);
+			max = MAX (i+1, 4096);
+		}
+		for (i = from; i < max; i++) {
+			if G_LIKELY (i != except && i != except2)
+				IGNORE_EINTR (close(i));
+		}
 	}
 }
 
@@ -1102,7 +1194,11 @@ gdm_open_dev_null (mode_t mode)
 {
 	int ret;
 	ret = open ("/dev/null", mode);
-	if (ret < 0) {
+	if G_UNLIKELY (ret < 0) {
+		/* never output anything, we're likely in some
+		 * strange state right now */
+		gdm_signal_ignore (SIGPIPE);
+		IGNORE_EINTR (close (2));
 		gdm_fail ("Cannot open /dev/null, system on crack!");
 	}
 
@@ -1117,15 +1213,45 @@ gdm_unset_signals (void)
 	sigemptyset (&mask);
 	sigprocmask (SIG_SETMASK, &mask, NULL);
 
-	signal (SIGUSR1, SIG_DFL);
-	signal (SIGUSR2, SIG_DFL);
-	signal (SIGCHLD, SIG_DFL);
-	signal (SIGTERM, SIG_DFL);
-	signal (SIGPIPE, SIG_DFL);
-	signal (SIGALRM, SIG_DFL);
-	signal (SIGHUP, SIG_DFL);
+	gdm_signal_default (SIGUSR1);
+	gdm_signal_default (SIGUSR2);
+	gdm_signal_default (SIGCHLD);
+	gdm_signal_default (SIGTERM);
+	gdm_signal_default (SIGINT);
+	gdm_signal_default (SIGPIPE);
+	gdm_signal_default (SIGALRM);
+	gdm_signal_default (SIGHUP);
 }
 
+void
+gdm_signal_ignore (int signal)
+{
+	struct sigaction ign_signal;
+
+	ign_signal.sa_handler = SIG_IGN;
+	ign_signal.sa_flags = SA_RESTART;
+	sigemptyset (&ign_signal.sa_mask);
+
+	if G_UNLIKELY (sigaction (signal, &ign_signal, NULL) < 0)
+		gdm_error (_("%s: Error setting signal %d to %s"),
+			   "gdm_signal_ignore", signal, "SIG_IGN");
+}
+
+void
+gdm_signal_default (int signal)
+{
+	struct sigaction def_signal;
+
+	def_signal.sa_handler = SIG_DFL;
+	def_signal.sa_flags = SA_RESTART;
+	sigemptyset (&def_signal.sa_mask);
+
+	if G_UNLIKELY (sigaction (signal, &def_signal, NULL) < 0)
+		gdm_error (_("%s: Error setting signal %d to %s"),
+			   "gdm_signal_ignore", signal, "SIG_DFL");
+}
+
+
 static char *
 ascify (const char *text)
 {
@@ -1229,25 +1355,117 @@ fillout_hostent (struct hostent *he_, struct in_addr *ia, const char *name)
 	return he;
 }
 
+static gboolean do_jumpback = FALSE;
+static Jmp_buf signal_jumpback;
+static struct sigaction oldterm, oldint, oldhup;
+
+static void
+jumpback_sighandler (int signal)
+{
+	/* this avoids a race see Note below.
+	   We want to jump back only on the first
+	   signal invocation, even if the signal
+	   handler didn't return. */
+	gboolean old_do_jumpback = do_jumpback;
+	do_jumpback = FALSE;
+
+	if (signal == SIGINT)
+		oldint.sa_handler (signal);
+	else if (signal == SIGTERM)
+		oldint.sa_handler (signal);
+	else if (signal == SIGHUP)
+		oldint.sa_handler (signal);
+	/* no others should be set up */
+
+	/* Note that we may not get here since
+	   the SIGTERM handler in slave.c
+	   might have in fact done the big Longjmp
+	   to the slave's death */
+	
+	if (old_do_jumpback) {
+		Longjmp (signal_jumpback, 1);
+	}
+}
+
+/*
+ * This sets up interruptes to be proxied and the
+ * gethostbyname/addr to be whacked using longjmp,
+ * in case INT/TERM/HUP was gotten in which case
+ * we no longer care for the result of the
+ * resolution.
+ */
+#define SETUP_INTERRUPTS_FOR_TERM_DECLS \
+    struct sigaction term;
+
+#define SETUP_INTERRUPTS_FOR_TERM_SETUP \
+    do_jumpback = FALSE;						\
+    									\
+    term.sa_handler = jumpback_sighandler;				\
+    term.sa_flags = SA_RESTART;						\
+    sigemptyset (&term.sa_mask);					\
+									\
+    if G_UNLIKELY (sigaction (SIGTERM, &term, &oldterm) < 0) 		\
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),	\
+		  "SETUP_INTERRUPTS_FOR_TERM", "TERM", strerror (errno)); \
+									\
+    if G_UNLIKELY (sigaction (SIGINT, &term, &oldint) < 0)		\
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),	\
+		  "SETUP_INTERRUPTS_FOR_TERM", "INT", strerror (errno)); \
+									\
+    if G_UNLIKELY (sigaction (SIGHUP, &term, &oldhup) < 0) 		\
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),	\
+		  "SETUP_INTERRUPTS_FOR_TERM", "HUP", strerror (errno)); \
+
+#define SETUP_INTERRUPTS_FOR_TERM_TEARDOWN \
+    do_jumpback = FALSE;						\
+									\
+    if G_UNLIKELY (sigaction (SIGTERM, &oldterm, NULL) < 0) 		\
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),	\
+		  "SETUP_INTERRUPTS_FOR_TERM", "TERM", strerror (errno)); \
+									\
+    if G_UNLIKELY (sigaction (SIGINT, &oldint, NULL) < 0) 		\
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),	\
+		  "SETUP_INTERRUPTS_FOR_TERM", "INT", strerror (errno)); \
+									\
+    if G_UNLIKELY (sigaction (SIGHUP, &oldhup, NULL) < 0) 		\
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),	\
+		  "SETUP_INTERRUPTS_FOR_TERM", "HUP", strerror (errno));
+
 GdmHostent *
 gdm_gethostbyname (const char *name)
 {
-	struct hostent *he_;
+	/* static because of Setjmp */
+	static struct hostent * he_;
+	SETUP_INTERRUPTS_FOR_TERM_DECLS
 
 	/* the cached address */
 	static GdmHostent *he = NULL;
 	static time_t last_time = 0;
 	static char *cached_hostname = NULL;
 
+	he_ = NULL;
+
 	if (cached_hostname != NULL &&
 	    strcmp (cached_hostname, name) == 0) {
-		/* don't check more then every 5 seconds */
-		if (last_time + 5 > time (NULL))
+		/* don't check more then every 60 seconds */
+		if (last_time + 60 > time (NULL))
 			return gdm_hostent_copy (he);
 	}
 
-	/* Find client hostname */
-	he_ = gethostbyname (name);
+	SETUP_INTERRUPTS_FOR_TERM_SETUP
+
+	if (Setjmp (signal_jumpback) == 0) {
+		do_jumpback = TRUE;
+		/* Find client hostname */
+		he_ = gethostbyname (name);
+		do_jumpback = FALSE;
+	} else {
+		/* here we got interrupted */
+		he_ = NULL;
+	}
+
+	SETUP_INTERRUPTS_FOR_TERM_TEARDOWN
+
 	g_free (cached_hostname);
 	cached_hostname = g_strdup (name);
 
@@ -1261,22 +1479,38 @@ gdm_gethostbyname (const char *name)
 GdmHostent *
 gdm_gethostbyaddr (struct in_addr *ia)
 {
-	struct hostent *he_;
+	/* static because of Setjmp */
+	static struct hostent * he_;
+	SETUP_INTERRUPTS_FOR_TERM_DECLS
 
 	/* the cached address */
 	static GdmHostent *he = NULL;
 	static time_t last_time = 0;
 	static struct in_addr cached_addr;
 
+	he_ = NULL;
+
 	if (last_time != 0 &&
 	    memcmp (&cached_addr, ia, sizeof (struct in_addr)) == 0) {
-		/* don't check more then every 5 seconds */
-		if (last_time + 5 > time (NULL))
+		/* don't check more then every 60 seconds */
+		if (last_time + 60 > time (NULL))
 			return gdm_hostent_copy (he);
 	}
 
-	/* Find client hostname */
-	he_ = gethostbyaddr ((gchar *) ia, sizeof (struct in_addr), AF_INET);
+	SETUP_INTERRUPTS_FOR_TERM_SETUP
+
+	if (Setjmp (signal_jumpback) == 0) {
+		do_jumpback = TRUE;
+		/* Find client hostname */
+		he_ = gethostbyaddr ((gchar *) ia, sizeof (struct in_addr), AF_INET);
+		do_jumpback = FALSE;
+	} else {
+		/* here we got interrupted */
+		he_ = NULL;
+	}
+
+	SETUP_INTERRUPTS_FOR_TERM_TEARDOWN
+
 	cached_addr = *ia;
 
 	gdm_hostent_free (he);
@@ -1328,8 +1562,15 @@ FILE *
 gdm_safe_fopen_w (const char *file)
 {
 	int fd;
-	unlink (file);
-	fd = open (file, O_EXCL|O_CREAT|O_TRUNC|O_WRONLY, 0644);
+	IGNORE_EINTR (unlink (file));
+	fd = open (file, O_EXCL|O_CREAT|O_TRUNC|O_WRONLY
+#ifdef O_NOCTTY
+		   |O_NOCTTY
+#endif
+#ifdef O_NOFOLLOW
+		   |O_NOFOLLOW
+#endif
+		   , 0644);
 	if (fd < 0)
 		return NULL;
 	return fdopen (fd, "w");
@@ -1342,70 +1583,288 @@ gdm_safe_fopen_ap (const char *file)
 	int fd;
 
 	if (access (file, F_OK) == 0) {
-#ifdef O_NOFOLLOW
-		fd = open (file, O_APPEND|O_RDWR|O_NOFOLLOW);
-#else
-		fd = open (file, O_APPEND|O_RDWR);
+		fd = open (file, O_APPEND|O_RDWR
+#ifdef O_NOCTTY
+			   |O_NOCTTY
 #endif
+#ifdef O_NOFOLLOW
+			   |O_NOFOLLOW
+#endif
+			  );
 	} else {
 		/* doesn't exist, open with O_EXCL */
-		fd = open (file, O_EXCL|O_CREAT|O_RDWR, 0644);
+		fd = open (file, O_EXCL|O_CREAT|O_RDWR
+#ifdef O_NOCTTY
+			   |O_NOCTTY
+#endif
+#ifdef O_NOFOLLOW
+			   |O_NOFOLLOW
+#endif
+			   , 0644);
 	}
 	if (fd < 0)
 		return NULL;
 	return fdopen (fd, "a+");
 }
 
+#ifdef RLIM_NLIMITS
+#define NUM_OF_LIMITS RLIM_NLIMITS
+#else /* ! RLIM_NLIMITS */
+#ifdef RLIMIT_NLIMITS
+#define NUM_OF_LIMITS RLIMIT_NLIMITS
+#endif /* RLIMIT_NLIMITS */
+#endif /* RLIM_NLIMITS */
+
+/* if we can count limits then the reset code is simple */ 
+#ifdef NUM_OF_LIMITS
+
+static struct rlimit limits[NUM_OF_LIMITS];
+
+void
+gdm_get_initial_limits (void)
+{
+	int i;
+
+	for (i = 0; i < NUM_OF_LIMITS; i++) {
+		/* some sane defaults */
+		limits[i].rlim_cur = RLIM_INFINITY;
+		limits[i].rlim_max = RLIM_INFINITY;
+		/* get the limits */
+		getrlimit (i, &(limits[i]));
+	}
+}
+
 void
 gdm_reset_limits (void)
 {
-	struct rlimit unlim = { RLIM_INFINITY, RLIM_INFINITY };
+	int i;
 
+	for (i = 0; i < NUM_OF_LIMITS; i++) {
+		/* get the limits */
+		setrlimit (i, &(limits[i]));
+	}
+}
+
+#else /* ! NUM_OF_LIMITS */
+/* we have to go one by one here */
+
+#ifdef RLIMIT_CPU
+static struct rlimit limit_cpu = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_DATA
+static struct rlimit limit_data = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_FSIZE
+static struct rlimit limit_fsize = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_LOCKS
+static struct rlimit limit_locks = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_MEMLOCK
+static struct rlimit limit_memlock = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_NOFILE
+static struct rlimit limit_nofile = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_OFILE
+static struct rlimit limit_ofile = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_NPROC
+static struct rlimit limit_nproc = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_RSS
+static struct rlimit limit_rss = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_STACK
+static struct rlimit limit_stack = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_CORE
+static struct rlimit limit_core = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_AS
+static struct rlimit limit_as = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_VMEM
+static struct rlimit limit_vmem = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+#ifdef RLIMIT_PTHREAD
+static struct rlimit limit_pthread = { RLIM_INFINITY, RLIM_INFINITY };
+#endif
+
+void
+gdm_get_initial_limits (void)
+{
 	/* Note: I don't really know which ones are really very standard
 	   and which ones are not, so I just test for them all one by one */
 
 #ifdef RLIMIT_CPU
-	setrlimit (RLIMIT_CPU, &unlim);
+	getrlimit (RLIMIT_CPU, &limit_cpu);
 #endif
 #ifdef RLIMIT_DATA
-	setrlimit (RLIMIT_DATA, &unlim);
+	getrlimit (RLIMIT_DATA, &limit_data);
 #endif
 #ifdef RLIMIT_FSIZE
-	setrlimit (RLIMIT_FSIZE, &unlim);
+	getrlimit (RLIMIT_FSIZE, &limit_fsize);
 #endif
 #ifdef RLIMIT_LOCKS
-	setrlimit (RLIMIT_LOCKS, &unlim);
+	getrlimit (RLIMIT_LOCKS, &limit_locks);
 #endif
 #ifdef RLIMIT_MEMLOCK
-	setrlimit (RLIMIT_MEMLOCK, &unlim);
+	getrlimit (RLIMIT_MEMLOCK, &limit_memlock);
 #endif
 #ifdef RLIMIT_NOFILE
-	setrlimit (RLIMIT_NOFILE, &unlim);
+	getrlimit (RLIMIT_NOFILE, &limit_nofile);
 #endif
 #ifdef RLIMIT_OFILE
-	setrlimit (RLIMIT_OFILE, &unlim);
+	getrlimit (RLIMIT_OFILE, &limit_ofile);
 #endif
 #ifdef RLIMIT_NPROC
-	setrlimit (RLIMIT_NPROC, &unlim);
+	getrlimit (RLIMIT_NPROC, &limit_nproc);
 #endif
 #ifdef RLIMIT_RSS
-	setrlimit (RLIMIT_RSS, &unlim);
+	getrlimit (RLIMIT_RSS, &limit_rss);
 #endif
 #ifdef RLIMIT_STACK
-	setrlimit (RLIMIT_STACK, &unlim);
+	getrlimit (RLIMIT_STACK, &limit_stack);
 #endif
 #ifdef RLIMIT_CORE
-	setrlimit (RLIMIT_CORE, &unlim);
+	getrlimit (RLIMIT_CORE, &limit_core);
 #endif
 #ifdef RLIMIT_AS
-	setrlimit (RLIMIT_AS, &unlim);
+	getrlimit (RLIMIT_AS, &limit_as);
 #endif
 #ifdef RLIMIT_VMEM
-	setrlimit (RLIMIT_VMEM, &unlim);
+	getrlimit (RLIMIT_VMEM, &limit_vmem);
 #endif
 #ifdef RLIMIT_PTHREAD
-	setrlimit (RLIMIT_PTHREAD, &unlim);
+	getrlimit (RLIMIT_PTHREAD, &limit_pthread);
 #endif
 }
 
+void
+gdm_reset_limits (void)
+{
+	/* Note: I don't really know which ones are really very standard
+	   and which ones are not, so I just test for them all one by one */
+
+#ifdef RLIMIT_CPU
+	setrlimit (RLIMIT_CPU, &limit_cpu);
+#endif
+#ifdef RLIMIT_DATA
+	setrlimit (RLIMIT_DATA, &limit_data);
+#endif
+#ifdef RLIMIT_FSIZE
+	setrlimit (RLIMIT_FSIZE, &limit_fsize);
+#endif
+#ifdef RLIMIT_LOCKS
+	setrlimit (RLIMIT_LOCKS, &limit_locks);
+#endif
+#ifdef RLIMIT_MEMLOCK
+	setrlimit (RLIMIT_MEMLOCK, &limit_memlock);
+#endif
+#ifdef RLIMIT_NOFILE
+	setrlimit (RLIMIT_NOFILE, &limit_nofile);
+#endif
+#ifdef RLIMIT_OFILE
+	setrlimit (RLIMIT_OFILE, &limit_ofile);
+#endif
+#ifdef RLIMIT_NPROC
+	setrlimit (RLIMIT_NPROC, &limit_nproc);
+#endif
+#ifdef RLIMIT_RSS
+	setrlimit (RLIMIT_RSS, &limit_rss);
+#endif
+#ifdef RLIMIT_STACK
+	setrlimit (RLIMIT_STACK, &limit_stack);
+#endif
+#ifdef RLIMIT_CORE
+	setrlimit (RLIMIT_CORE, &limit_core);
+#endif
+#ifdef RLIMIT_AS
+	setrlimit (RLIMIT_AS, &limit_as);
+#endif
+#ifdef RLIMIT_VMEM
+	setrlimit (RLIMIT_VMEM, &limit_vmem);
+#endif
+#ifdef RLIMIT_PTHREAD
+	setrlimit (RLIMIT_PTHREAD, &limit_pthread);
+#endif
+}
+
+#endif /* NUM_OF_LIMITS */
+
+const char *
+gdm_root_user (void)
+{
+	static char *root_user = NULL;
+	struct passwd *pwent;
+
+	if (root_user != NULL)
+		return root_user;
+
+	pwent = getpwuid (0);
+	if (pwent == NULL) /* huh? */
+		root_user = g_strdup ("root");
+	else
+		root_user = g_strdup (pwent->pw_name);
+	return root_user;
+}
+
+void
+gdm_sleep_no_signal (int secs)
+{
+	time_t endtime = time (NULL)+secs;
+
+	while (secs > 0) {
+		struct timeval tv;
+		tv.tv_sec = secs;
+		tv.tv_usec = 0;
+		select (0, NULL, NULL, NULL, &tv);
+		/* don't want to use sleep since we're using alarm
+		   for pinging */
+		secs = endtime - time (NULL);
+	}
+}
+
+char *
+gdm_make_filename (const char *dir, const char *name, const char *extension)
+{
+	char *base = g_strconcat (name, extension, NULL);
+	char *full = g_build_filename (dir, base, NULL);
+	g_free (base);
+	return full;
+}
+
+char *
+gdm_ensure_extension (const char *name, const char *extension)
+{
+	const char *p;
+
+	if (ve_string_empty (name))
+		return g_strdup (name);
+
+	p = strrchr (name, '.');
+	if (p != NULL &&
+	    strcmp (p, extension) == 0) {
+		return g_strdup (name);
+	} else {
+		return g_strconcat (name, extension, NULL);
+	}
+}
+
+char *
+gdm_strip_extension (const char *name, const char *extension)
+{
+	const char *p = strrchr (name, '.');
+	if (p != NULL &&
+	    strcmp (p, extension) == 0) {
+		char *r = g_strdup (name);
+		char *rp = strrchr (r, '.');
+		*rp = '\0';
+		return r;
+	} else {
+		return g_strdup (name);
+	}
+}
+
 /* EOF */
