@@ -65,7 +65,7 @@ static void
 query_greeter_halt_handler (void)
 {
 	if (greeter_query (_("Are you sure you want to shut down the machine?"),
-			   _("Shut _Down"), GTK_STOCK_CANCEL)) {
+			   _("Shut_down"), GTK_STOCK_CANCEL)) {
 		closelog();
 
 		_exit (DISPLAY_HALT);
@@ -139,7 +139,7 @@ greeter_system_append_system_menu (GtkWidget *menu)
 		tooltips = gtk_tooltips_new ();
 
 	if (GdmChooserButton) {
-		w = gtk_menu_item_new_with_mnemonic (_("Run _XDMCP Chooser"));
+		w = gtk_menu_item_new_with_mnemonic (_("_XDMCP Chooser..."));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 		gtk_widget_show (GTK_WIDGET (w));
 		g_signal_connect (G_OBJECT (w), "activate",
@@ -154,7 +154,7 @@ greeter_system_append_system_menu (GtkWidget *menu)
 
 	if (GdmConfigAvailable &&
 	    bin_exists (GdmConfigurator)) {
-		w = gtk_menu_item_new_with_mnemonic (_("_Configure the login manager..."));
+		w = gtk_menu_item_new_with_mnemonic (_("_Configure Login Manager..."));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 		gtk_widget_show (GTK_WIDGET (w));
 		g_signal_connect (G_OBJECT (w), "activate",
@@ -177,7 +177,7 @@ greeter_system_append_system_menu (GtkWidget *menu)
 	}
 
 	if (add_reboot) {
-		w = gtk_menu_item_new_with_mnemonic (_("_Reboot the computer..."));
+		w = gtk_menu_item_new_with_mnemonic (_("_Reboot"));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 		gtk_widget_show (GTK_WIDGET (w));
 		g_signal_connect (G_OBJECT (w), "activate",
@@ -189,20 +189,20 @@ greeter_system_append_system_menu (GtkWidget *menu)
 	}
 
 	if (add_halt) {
-		w = gtk_menu_item_new_with_mnemonic (_("Shut _down the computer..."));
+		w = gtk_menu_item_new_with_mnemonic (_("Shut_down"));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 		gtk_widget_show (GTK_WIDGET (w));
 		g_signal_connect (G_OBJECT (w), "activate",
 				  G_CALLBACK (query_greeter_halt_handler),
 				  NULL);
 		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (w),
-				      _("Shut down your computer so that "
-					"you may turn it off."),
+				      _("Shut down the system so that "
+					"you may safely turn off the computer."),
 				      NULL);
 	}
 
 	if (add_suspend) {
-		w = gtk_menu_item_new_with_mnemonic (_("Sus_pend the computer..."));
+		w = gtk_menu_item_new_with_mnemonic (_("Sus_pend"));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 		gtk_widget_show (GTK_WIDGET (w));
 		g_signal_connect (G_OBJECT (w), "activate",
@@ -393,15 +393,15 @@ greeter_system_handler (GreeterItemInfo *info,
       return;
     }
 
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (halt_radio)))
+  if (halt_radio != NULL && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (halt_radio)))
     greeter_halt_handler();
-  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (reboot_radio)))
+  else if (reboot_radio != NULL && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (reboot_radio)))
     greeter_reboot_handler ();
-  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (suspend_radio)))
+  else if (suspend_radio != NULL && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (suspend_radio)))
     greeter_suspend_handler ();
-  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (config_radio)))
+  else if (config_radio != NULL && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (config_radio)))
     greeter_config_handler ();
-  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (chooser_radio)))
+  else if (chooser_radio != NULL && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (chooser_radio)))
     greeter_chooser_handler ();
 
   gtk_widget_destroy (dialog);
